/**
 * @file main.c
 * @author Jake G
 * @date 15 June 2024
 * @brief File contains the main function.
 *
 * This file contains the main logic loop and exists to setup the values
 * specific to the micro-controller. All other functions and configuration are
 * extracted into separate source files and headers for configuration.
 */

#include <stdio.h>
#define F_CPU 3333333UL

// This can prevent issues with utils/delay.h library with the gcc toolchain
#define __DELAY_BACKWARD_COMPATIBLE__

#include "RegEdit.h"
#include "config.h"
#include "timer.h"
#include <avr/cpufunc.h> /* Required header file */
#include <avr/io.h>
#include <util/delay.h>

#define SW1PIN (1<<5)
#define SW2PIN (1<<4)
#define RELAYPIN (1<<1)
#define RELAYREADINGPIN (1<<7) //It would be better to use PA7 so USART worked


//Set the function pointer for the delay func
void (*Delay_MicroSeconds)(double us) = _delay_us;

void SW1_Wait(void)
{ 
    //poll the input.
    while(PORTA.IN & SW1PIN){}
}

void SW2_Wait(void)
{ 
    //poll the input.
    while(PORTA.IN & SW2PIN){}
}


void Activate_Relay(void)
{
    PORTA.OUT |= RELAYPIN;
}

void Deactivate_Relay(void)
{
    PORTA.OUT &= ~RELAYPIN;
}


void WaitForRelayConnect(void)
{
    while(!(PORTA.IN & RELAYREADINGPIN)){;}
}

void WaitForRelayDisconnect(void)
{
    while(PORTA.IN & RELAYREADINGPIN){;}
}


int main(int argc, char **argv)
{
    PORTA.DIR |= RELAYPIN;
    PORTA.DIR &= ~RELAYREADINGPIN;
    PORTA.DIR &= ~SW1PIN;
    PORTA.DIR &= ~SW2PIN;


    uint16_t make_time = 0;
    uint16_t break_time = 0;
    

    while(true) {
        SW1_Wait();
        Activate_Relay();

        Timer_Start();
        WaitForRelayConnect();
        Timer_Disable(); 

        make_time = Timer_GetOverflowCount();


        //Output the Make time via LEDS

        SW2_Wait();
        Deactivate_Relay();

        Timer_Start();
        WaitForRelayDisconnect();
        Timer_Disable();

        break_time = Timer_GetOverflowCount();

        //Output the Break time via LEDS

        /*
        USART0_sendString(maketime_msg);
        USART0_sendChar((uint8_t)(0xFF & make_time)); 
        USART0_sendString(breaktime_msg);
        USART0_sendChar((uint8_t)(0xFF & break_time)); 
        */
    }
}
